/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "widgetprincipal.h"

#include <QMenuBar>
#include <QAction>
#include <QLayout>
#include <QTextEdit>
#include <QLabel>
#include <QLCDNumber>
#include <QPixmap>
#include <QPushButton>
#include <QFile>
#include <QTextStream>
#include <QColor>
#include <QStatusBar>
#include <QTime>
#include <QTimer>
#include <QCursor>

//Cration d'un widget regroupant 1 Lcd et 2 pushbuttons pour les plaques
WidgetPlaque::WidgetPlaque( QWidget *parent, int numW, bool est_compteW)
        : QWidget( parent) {
	
	//init du num de widget et de variable tat est_compte
	num=numW;
	est_compte=est_compteW;
	
	//init des elements graphiques
	vboxLayout = new QVBoxLayout(this);
	vboxLayout->setSpacing(0);
	vboxLayout->setMargin(0);
	
	//un afficheur lcd
	lcd = new QLCDNumber(this);
	
	if (est_compte) {
		lcd->setNumDigits( 1 );
		lcd->setFixedSize( 32, 51 );
	} else {
		lcd->setNumDigits( 3 );
		lcd->setFixedHeight( 28 );
	}
	
	lcd->setSegmentStyle( QLCDNumber::Flat );
	lcd->setAutoFillBackground ( true );
	
	//couleurs, polices qui serviront en plusieurs endroits par la suite
	palette_orange = new QPalette();
	palette_orange->setColor( QPalette::Window, QColor( 255, 196, 128 ) );
	palette_blanc = new QPalette();
	palette_blanc->setColor( QPalette::Window, QColor( 255, 255, 255 ) );
	//deux pushbuttons dans un layout horizontal
	QString chMoins;
	QString chPlus;
	
	if (est_compte) {
		lcd->setPalette( *palette_orange );
		chMoins=QString(":/images/moins.png");
		chPlus=QString(":/images/plus.png");
	} else {
		lcd->setPalette( *palette_blanc );
		chMoins=QString(":/images/moins2.png");
		chPlus=QString(":/images/plus2.png");
	}
	
	QPixmap *pixMoins = new QPixmap( chMoins );
	QPixmap *pixPlus = new QPixmap( chPlus );
	
	hbox = new QWidget(this);
	hboxLayout = new QHBoxLayout(hbox);
	hboxLayout->setSpacing(0);
	hboxLayout->setMargin(0);
	
	pbMoins = new QPushButton( "", hbox);
	pbMoins->setFlat( true );
	pbMoins->setIcon( *pixMoins );
	pbMoins->setFixedSize( 15, 15 );
	
	pbPlus = new QPushButton( "", hbox);
	pbPlus->setFlat( true );
	pbPlus->setIcon( *pixPlus );
	pbPlus->setFixedSize( 15, 15 );
	
	hboxLayout->addWidget( pbMoins);
	hboxLayout->addWidget( pbPlus);
	
	vboxLayout->addWidget(lcd);
	vboxLayout->addWidget(hbox);
	
	//connections internes
	connect( pbMoins, SIGNAL(clicked()), this, SLOT(pbMoinsClicked()) );
	connect( pbPlus, SIGNAL(clicked()), this, SLOT(pbPlusClicked()) );
}

void WidgetPlaque::pbMoinsClicked() {
	emit change(num, FALSE);
}

void WidgetPlaque::pbPlusClicked() { 
	emit change(num, TRUE);
}

void WidgetPlaque::displayRefresh(int numW, int valeur) {
	//rception du signal si le numro de widget correspond
	//mthode un peu lourde mais qui permet de supprimer le nom des des widgets
	if(num==numW) {
		lcd->display(valeur);
	}
}


//---------------------Widget Principal----------------------------------------

Principal::Principal( QWidget *parent ) : QMainWindow( parent ) {
	
	//polices de caractres
	QFont font_normal, font_monospace, font_gros;
	font_normal.setFamily( "Bitstream Vera Sans" );
	font_monospace.setFamily( "Bitstream Vera Sans Mono" );
	font_gros.setFamily( "Bitstream Vera Sans" );
	
	#ifdef LINUX
		font_normal.setPointSize( 12 );
		font_monospace.setPointSize( 12 );
		font_gros.setPointSize( 16 );
	#endif
	#ifdef WINDOWS
		font_normal.setPointSize( 10 );
		font_monospace.setPointSize( 10 );
		font_gros.setPointSize( 14 );
	#endif
	
	font_gros.setBold( TRUE );
	
	//Barre de menus
	MenuBar = new QMenuBar( this );
	setMenuBar(MenuBar);
	MenuBar->setFont( font_normal );
	
	fichierMenu = new QMenu ( this );
	infoMenu = new QMenu ( this );
	
	fileExitAction = new QAction( this );
	fichierMenu = menuBar()->addMenu("&Fichier");
	fichierMenu->addAction(fileExitAction);
	fichierMenu->setFont( font_normal );
	
	helpAideAction = new QAction( this );
	helpLicenceAction = new QAction( this );
	helpAboutAction = new QAction( this );
	infoMenu = menuBar()->addMenu("&Info");
	infoMenu->setFont( font_normal );
	infoMenu->addAction(helpAideAction);
	infoMenu->addAction(helpLicenceAction);
	infoMenu->addAction(helpAboutAction);
	
	//Pour la barre de statut, temps de calcul
	heure = new QTime();
	
	//Widget parent
	hbox = new QWidget( this );
	hboxLayout = new QHBoxLayout( hbox );
	
	hboxLayout->setSpacing(1);
	hboxLayout->setMargin(1);
	
	setCentralWidget(hbox);
	
	//Bote verticale contenant tous les lments de la gauche de la fentre
	vboxA = new QWidget( hbox );
	vboxLayoutA = new QVBoxLayout( vboxA );
	
	vboxA->setFixedWidth(288);
	vboxLayoutA->setSpacing(0);
	vboxLayoutA->setMargin(0);
	
	//cadre de texte pour les rsultats, colonne de droite
	textEditB = new QTextEdit( hbox );
	textEditB->setReadOnly( TRUE );
	#ifdef LINUX
		textEditB->setFixedWidth(260);
	#endif
	#ifdef WINDOWS
		textEditB->setFixedWidth(280);
	#endif
	textEditB->setFont( font_monospace );
	
	//insertion des widgets dans le layout
	hboxLayout->addWidget( vboxA );
	hboxLayout->addWidget( textEditB );
	
	//Widgets aligns verticalement  gauche
	horLayout1 = new QWidget( vboxA );
	hboxLayout1 = new QHBoxLayout(horLayout1);
	hboxLayout1->setSpacing(0);
	hboxLayout1->setMargin(0);
	horLayout2 = new QWidget( vboxA );
	hboxLayout2 = new QHBoxLayout(horLayout2);
	hboxLayout2->setSpacing(0);
	hboxLayout2->setMargin(0);
	horLayout3 = new QWidget( vboxA );
	hboxLayout3 = new QHBoxLayout(horLayout3);
	hboxLayout3->setSpacing(0);
	hboxLayout3->setMargin(0);
	horLayout4 = new QWidget( vboxA );
	hboxLayout4 = new QHBoxLayout(horLayout4);
	hboxLayout4->setSpacing(0);
	hboxLayout4->setMargin(0);
	texte7 = new QLabel( vboxA );
	BarreStatut = new QStatusBar ( vboxA );
	
	//Espaces qui servent plus loin pour placer ou centrer les lments
	spacerItem1 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
	spacerItemV = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
	
	//insertion des widgets dans le layout
	vboxLayoutA->addItem(spacerItemV);
	vboxLayoutA->addWidget( horLayout1 );
	vboxLayoutA->addWidget( horLayout2 );
	vboxLayoutA->addWidget( horLayout3 );
	vboxLayoutA->addWidget( horLayout4 );
	vboxLayoutA->addWidget( texte7 );
	vboxLayoutA->addWidget( BarreStatut );
	vboxLayoutA->addItem(spacerItemV);
	
	//premire ligne : le compte + les modifieurs
	//WidgetPlaque 10, 11 et 12
	hboxLayout1->addItem( spacerItem1 );
	
	for(int Wi=0; Wi<3; Wi++) {
		WidgetPlaque *plaque= new WidgetPlaque(horLayout1, Wi+10, true);
		hboxLayout1->addWidget( plaque );
		connect( this, SIGNAL(refresh(int, int)), plaque, SLOT(displayRefresh(int, int)) );
		connect( plaque, SIGNAL(change(int, bool)), this, SLOT(compte_change(int, bool)) );
	}
	
	hboxLayout1->addItem( spacerItem1 );
	
	//deuxime ligne : les plaques + les modifieurs
	horLayout2->setFixedHeight( 43 );
	
	//on "oublie" les widgets crs (le moteur Qt ne les oublie pas) et on ne communique que par signaux
	//conformment au tutorial Qt4 "Qt Tutorial 7"
	for(int Wi=0; Wi<6; Wi++) {
		WidgetPlaque *plaque= new WidgetPlaque(horLayout2, Wi, false);
		hboxLayout2->addWidget( plaque );
		connect( this, SIGNAL(refresh(int, int)), plaque, SLOT(displayRefresh(int, int)) );
		connect( plaque, SIGNAL(change(int, bool)), this, SLOT(plaque_change(int, bool)) );
	}
	
	//troisime ligne : les boutons
	horLayout3->setFixedHeight( 27 );
	
	pbRes = new QPushButton( "Rsoudre", horLayout3 );
	pbAlea = new QPushButton( "Alatoire", horLayout3 );
	pbZero = new QPushButton( "Mise  zro", horLayout3 );
	
	pbRes->setFont( font_normal );
	pbAlea->setFont( font_normal );
	pbZero->setFont( font_normal );
	
	hboxLayout3->addWidget( pbRes );
	hboxLayout3->addWidget( pbAlea );
	hboxLayout3->addWidget( pbZero );
	
	//quatrime ligne : image du renard
	horLayout4->setFixedHeight(150);
	
	pixmapLabel6_1 = new QLabel( horLayout4 );
	renard_normal = new QPixmap(":/images/normal-150x150.png");
	renard_heureux = new QPixmap(":/images/heureux-150x150.png");
	renard_triste = new QPixmap(":/images/triste-150x150.png");
	pixmapLabel6_1->setPixmap(*renard_normal);
	pixmapLabel6_1->setFixedSize( 150, 150 );
	
	hboxLayout4->addItem(spacerItem1);
	hboxLayout4->addWidget(pixmapLabel6_1);
	hboxLayout4->addItem(spacerItem1);
	
	//cinquime ligne : le texte
	texte7->setFont( font_gros );
	texte7->setAlignment( Qt::AlignHCenter );
	texte7->setText( "Kitsune - v1.0 - Bonjour !" );
	
	//sixime ligne : la barre de statut
	BarreStatut->setSizeGripEnabled ( false );
	BarreStatut->setFont( font_normal );
	
	//initialisation des paramtres
	//"demarrage" sert  ne pas modifier le texte d'accueil,  l'ouverture de l'application
	demarrage=true;
	mise_a_zero();
	demarrage=false;
	
	//pour avoir du vrai alatoire, pas toujours les mmes valeurs
	srand (time (NULL));
	
	//initialisation des menus
	fileExitAction->setText( "&Quitter" );
	helpAideAction->setText( "Ai&de" );
	helpLicenceAction->setText( "&Licence" );
	helpAboutAction->setText( "A &propos" ); 
	
	//fonctions de connexion signal<--->slot
	//fonctions pour la barre de menus
	connect( fileExitAction, SIGNAL( triggered() ), this, SLOT( close() ) );
	connect( helpAideAction, SIGNAL( triggered() ), this, SLOT( helpAide() ) );
	connect( helpLicenceAction, SIGNAL( triggered() ), this, SLOT( helpLicence() ) );
	connect( helpAboutAction, SIGNAL( triggered() ), this, SLOT( helpAbout() ) );
	
	//fonctions pour les boutons
	connect( pbRes, SIGNAL(clicked()), this, SLOT(resoudre1()) );
	connect( pbAlea, SIGNAL(clicked()), this, SLOT(aleatoire()) );
	connect( pbZero, SIGNAL(clicked()), this, SLOT(mise_a_zero()) );
}

//fonction d'ouverture de la fentre d'aide
void Principal::helpAide() {
	lire_texte(2);
}

//fonction d'ouverture de la fentre de license
void Principal::helpLicence() {
	lire_texte(1);
}

//fonction d'ouverture de la fentre " propos"
void Principal::helpAbout() {
	lire_texte(0);
}

void Principal::lire_texte(int num) {
	bool license=false;
	if (num==1) license=true;
	dlgPopup dlg( this, license );
	
	QString chAbout;
	if (num==0) {
		chAbout=QString(":/txt/about.txt");
	} else if (num==1) {
		chAbout=QString(":/txt/licence.txt");
	} else if (num==2) {
		chAbout=QString(":/txt/aide.txt");
	}
	
	QFile gpl( chAbout );
	gpl.open( QIODevice::ReadOnly );
	QTextStream gplstream( &gpl );
	dlg.textEditPopup->setPlainText(gplstream.readAll());
	gpl.close();
	
	dlg.exec();
}

//une fonction pour mettre  jour l'affichage des plaques
void Principal::update_affichage_lcd3() {
	for(int Wi=0; Wi<6; Wi++) {
		emit refresh(Wi, liste_des_plaques[Wi]);
	}
}

//une fonction pour mettre  jour l'affichage du compte
void Principal::update_affichage_compte() {
	for(int Wi=0; Wi<3; Wi++) {
		emit refresh(Wi+10, liste_des_chiffres[Wi]);
	}
}

//une fonction pour mettre  jour l'affichage du statut, du renard et du texte
void Principal::update_affichage_statut() {
	BarreStatut->showMessage( texte_statut_attente );
	pixmapLabel6_1->setPixmap(*renard_normal);
	texte7->setText( texte_gros_attente );
}


//seul moyen trouv pour l'instant pour que le "Calcul en cours..." s'affiche
//si je ne mets pas le singleshot, il semble qu'il rsolve le problme avant d'afficher le message, et ce message se fait
//immdiatement craser  la fin par le suivant qui indique la dure du calcul
void Principal::resoudre1() {
	BarreStatut->showMessage( QString("Calcul en cours...") );
	setCursor( QCursor(Qt::WaitCursor));		//la souris est un sablier pendant le calcul
	QTimer::singleShot(1, this, SLOT(resoudre2()));
}


//rsolution du problme, et affichage dans le cadre de droite
void Principal::resoudre2() {
	heure->start();
	
	jeu J;
	//calcul du compte  tudier
	compte le_compte_etudie=liste_des_chiffres[0]*100+liste_des_chiffres[1]*10+liste_des_chiffres[2];
	//tude du tirage
	J.etude_tirage(t,le_compte_etudie,le_compte_etudie);
	//affichage des rsultats dans le cadre de texte prvu  cet effet
	textEditB->setPlainText(QString(J.afficheBase().c_str()));
	//petite phrase d'information en bas de la fentre + image
	if(J.distance_a_la_solution()==0){
		texte7->setText( "Le compte est bon !" );
		pixmapLabel6_1->setPixmap(*renard_heureux);
	}
	else{
		texte7->setText( "Il n'y a pas de solution." );
		pixmapLabel6_1->setPixmap(*renard_triste);
	}
	int temps_ecoule=heure->elapsed();
	int secondes=temps_ecoule/1000;
	int centiemes=(temps_ecoule%1000)/10;
	QString resultat=QString("Dure du calcul : ");
	resultat.append( QString().sprintf("%d,", secondes) );
	resultat.append( QString().sprintf("%d s", centiemes) );
	
	setCursor( QCursor(Qt::ArrowCursor));	//une fois le calcul termin le pointeur de la souris redevient une flche
	BarreStatut->showMessage( resultat );
}

//dtermination alatoire d'un nouveau problme
void Principal::aleatoire(){
	//dtermination du nouveau compte
	liste_des_chiffres.clear();
	liste_des_chiffres.assign(3,0);
	
	liste_des_chiffres[0]=1+rand()%9;
	liste_des_chiffres[1]=rand()%10;
	liste_des_chiffres[2]=rand()%10;
	
	//affichage du nouveau compte
	update_affichage_compte();
	
	//dtermination des plaques et du tirage
	t.clear();
	
	pseudo_tirage pt;  //pseudo-tirage pour le calcul alatoire des plaques
	pt.assign(6,0);
	liste_des_plaques.clear();
	liste_des_plaques.assign(6,0);
	int i;
	int hasard;
	vector<int> v;
	v.assign(24,0);
	for(i=0;i<=23;i++){
		v[i]=i;
	}

	for(i=0;i<6;i++){
		hasard=rand()%(24-i);
		pt[i]=v[hasard];
		v.erase(v.begin()+hasard);
		switch (pt[i]){
			case 23: liste_des_plaques[i]=100; break;
			case 22: liste_des_plaques[i]=75; break;
			case 21: liste_des_plaques[i]=50; break;
			case 20: liste_des_plaques[i]=25; break;
			default: liste_des_plaques[i]=(1+pt[i]/2); break;
		}
	}
	t=pseudo_en_tirage(pt);
	
	//affichage des plaques et statut
	update_affichage_lcd3();
	update_affichage_statut();
}

//remise de l'application dans l'tat initial (hormis le texte d'accueil)
void Principal::mise_a_zero(){
	//initialisation des textes et images
	if(!demarrage){
		texte7->setText( texte_gros_attente );
		pixmapLabel6_1->setPixmap(*renard_normal);
	}
	textEditB->setPlainText("");
	BarreStatut->showMessage( texte_statut_attente );
	
	//initialisation du compte
	liste_des_chiffres.clear();
	liste_des_chiffres.assign(3,0);
	liste_des_chiffres[0]=1;
	liste_des_chiffres[1]=0;
	liste_des_chiffres[2]=0;
	
	//affichage du compte
	update_affichage_compte();
	
	//initialisation des plaques
	int i;
	liste_des_plaques.clear();
	liste_des_plaques.assign(6,0);
	for(i=0;i<6;i++){
		liste_des_plaques[i]=i+1;
	}
	
	//affichage des plaques
	update_affichage_lcd3();
	
	//initialisation du tirage
	t.clear();
	for(i=1;i<=6;i++){
		t[i]++;
	}
}

//fonction appele  chaque modification du compte
void Principal::compte_change(int numero_plaque, bool est_plus) {
	if(est_plus) {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]+1;
	} else {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]-1;
	}

	if(liste_des_chiffres[0]>=1 && 
	   liste_des_chiffres[numero_plaque-10]>=0 &&
	   liste_des_chiffres[numero_plaque-10]<=9
           ) {
		update_affichage_compte();
		update_affichage_statut();
		return;
	}

	//annulation de la modif
	if(est_plus) {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]-1;
	} else {
		liste_des_chiffres[numero_plaque-10]=liste_des_chiffres[numero_plaque-10]+1;
	}
}

//fonction appele  chaque modification d'une plaque
void Principal::plaque_change(int numero_plaque, bool est_plus) {
	plaque plaque_passee=liste_des_plaques[numero_plaque];
	plaque plaque_future=0;
	if(est_plus){
		plaque_future=plus_petite_plaque_sup(t,plaque_passee);
	} else{
		plaque_future=plus_petite_plaque_inf(t,plaque_passee);
	}
	
	if(plaque_future!=0){
		liste_des_plaques[numero_plaque]=plaque_future;
		t[plaque_passee]--;
		t[plaque_future]++;
	
		//affichage
		update_affichage_lcd3();
		update_affichage_statut();  
	}
}
